import { css } from "@linaria/core";
import { defineComponent, onMounted, ref } from "vue";
import { string } from "vue-types";
import { useEditor } from "../../../..";
import RingBtns from "./RingBtns";

export const Transfer = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props) {
    const { store, helper } = useEditor();
    const resizeRef = ref<HTMLElement>();
    const moveRef = ref<HTMLElement>();
    const start = {
      x: 0,
      y: 0,
      width: 0,
      height: 0,
      offsetX: 0,
      offsetY: 0,
    };

    const comp = store.designData.compMap[props.compId];
    let compEl: HTMLElement;

    onMounted(() => {
      if (!resizeRef.value || !moveRef.value) return;
      resizeRef.value.addEventListener("mousedown", startDrag);
      moveRef.value.addEventListener("mousedown", startMove);
      compEl = moveRef.value.parentElement as HTMLElement;
    });

    function startMove(e: MouseEvent) {
      start.x = e.clientX;
      start.y = e.clientY;

      document.addEventListener("mousemove", move);
      document.addEventListener("mouseup", stopMove);
    }
    function move(e: MouseEvent) {
      const viewEl = compEl.parentElement;
      if (!viewEl) return;
      start.offsetX = e.clientX - start.x;
      start.offsetY = e.clientY - start.y;

      viewEl.style.left = helper.designToNaturalSize(
        comp.value.position[0] + start.offsetX * 2
      );
      viewEl.style.top = helper.designToNaturalSize(
        comp.value.position[1] + start.offsetY * 2
      );
    }
    function stopMove(e: MouseEvent) {
      comp.value.position[0] += start.offsetX * 2;
      comp.value.position[1] += start.offsetY * 2;
      document.removeEventListener("mousemove", move);
      document.removeEventListener("mouseup", stopMove);
    }

    function startDrag(e: MouseEvent) {
      start.x = e.clientX;
      start.y = e.clientY;
      start.width = comp.layout.size?.[0] || compEl.clientWidth * 2;
      start.height = comp.layout.size?.[1] || compEl.clientHeight * 2;

      document.addEventListener("mousemove", drag);
      document.addEventListener("mouseup", stopDrag);
    }

    function drag(e: MouseEvent) {
      console.log(e.clientX, e.clientY);
      start.offsetX = e.clientX - start.x;
      start.offsetY = e.clientY - start.y;
      compEl.style.width = helper.designToNaturalSize(
        start.width + start.offsetX * 2
      );
      compEl.style.height = helper.designToNaturalSize(
        start.height + start.offsetY * 2
      );
    }

    function stopDrag() {
      comp.layout.size || (comp.layout.size = [0, 0]);
      comp.layout.size[0] = start.width + start.offsetX * 2;
      comp.layout.size[1] = start.height + start.offsetY * 2;
      document.removeEventListener("mousemove", drag);
      document.removeEventListener("mouseup", stopDrag);
    }

    return () => (
      <>
        <div class={borderStyle}></div>
        <div ref={resizeRef} class={[resizeStyle, "drag-disable"]}></div>
        <div ref={moveRef} class={[moveStyle, "drag-disable"]}></div>
        <RingBtns class={offsetStyle} />
      </>
    );
  },
});

const borderStyle = css`
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  outline: 2px solid @inf-primary-color;
  pointer-events: none;
  z-index: 9;
`;

const resizeStyle = css`
  position: absolute;
  bottom: 0;
  right: 0;
  width: 16px;
  height: 16px;
  border-radius: 50%;
  background-color: #fff;
  z-index: 9;
  transform: translate(50%, 50%);
  box-shadow: 0 0 2px 2px rgba(0, 0, 0, 0.2);
  cursor: nwse-resize;
  &:hover {
    border-color: @inf-primary-color;
  }
`;

const moveStyle = css`
  position: absolute;
  bottom: 0;
  left: 50%;
  width: 40px;
  height: 8px;
  transform: translate(-50%, 5px);
  background-color: #fff;
  border-radius: 8px;
  box-shadow: 0 0 2px 1px rgba(0, 0, 0, 0.2);
  z-index: 9;
  cursor: all-scroll;
`;

const offsetStyle = css`
  position: absolute;
  bottom: 0;
  left: 50%;
  transform: translate(-50%, 60px);
  z-index: 9;
`;
